<?php // $Id: lilie.php 399 2011-10-12 15:22:35Z fl $
/**
 * IMPORTATION LILIE
 * 
 * Importe dans une base MySQL l'extraction standard : "listeEleves.csv"
 *
 * @author Jean-Marc L. <Jean-Marc.Labat@ac-creteil.fr>
 * @author François L. <FL@ac-creteil.fr>
 * @version 0.9
 * @package SCONET
 */

class Enteleves {
	
	/**
	* RNE de l'établissement
	* @var String	
	*/
	public $RNE;
	
	/**
	* Tableau des élèves de l'établissement RNE
	* @var array
	*/
	public $eleves = array(); 

	/**
	* Tableau des classes de l'établissement RNE
	* @var array
	*/
	public $classes = array();

	/**
	* Modèle de données SQL
	* @var Modele
	*/
	public $modele;
	
	public $classe;
	/**
	* Constructeur 
	*
	* Initialisation des élèves ou des classes depuis un tableau ($csv)  
	*
	* @param string $rne
	* @param string $csv
	* @param lilieModele &$modele
	*/
	public function Enteleves( $elevesCSV, $modele,$classe,$rne){
		$args = get_defined_vars();

		$this->RNE = $rne;
		$this->eleves = $elevesCSV;
		$this->modele = $modele;

		////__LXF__(__CLASS__."::".__FUNCTION__, $args, $this );
	}
	
	/**
	* Tableau des eleves sortants
	* Difference $this->elevesEtablissement - $this->eleves
	*
	* @return array
	*/
	private function elevesAnciens(){
		$args = get_defined_vars();

		$Ei = $this->modele->sconetEleves($this->RNE);
		$Ef = $this->eleves;
		$return = array_diff_key($Ei,$Ef);
		Modele::__DEBUG__($return);
		////__LXF__(__CLASS__."::".__FUNCTION__, $args, $return );
		return $return;
	}
	
	/**
	* Insère les élèves et les classes dans la base de données
	*
	* Vide les tables au préalable
	*
	* TRANSACTION - Nécessite que le moteur de stockage des tables Sconet soient en InnoDB
	*
	* @see $modele->PDO, $modele->videTableSconet
	* @param boolean $debug 
	* @return string
	* 
	*/
	public function toMySQL2( $trans = true ){
		$args = get_defined_vars();

		$html = ""; //"<h1>".$this->RNE."</h1>";
		
		// Gestionnaire d'erreur en mode exception
		$this->modele->PDO->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		
		// Débuter la transaction 
		$this->modele->PDO->beginTransaction();
		
		try { // on essaye ...
		 	//$this->modele->videTablesSconet($this->RNE);
			Modele::__DEBUG__("commence transaction");
			
			foreach($this->eleves as $E){
				Modele::__DEBUG__("eleve A creer");
				Modele::__DEBUG__($E);
				$E['id_cas']=$E['id'];

				if(!isset($E['id_sconet'])){

					$E['id_sconet']=$E['id'];
					
				}
				if(!isset($E['login'])){
					$E['login'] = $this->getLogin($E['prenom'], $E['nom']);
				}
				if(isset($E['naissance'])){


					$parts = explode('/', $E['naissance']);
					if(count($parts)==3){
						$date  = "$parts[2]-$parts[1]-$parts[0]";
						$E['naissance'] = $date;
					}
					
				}

				
				
				$html .= $this->modele->storeEleve($E, 'sconet');

				$html .= $this->modele->storeEleveSiBalado($E, 'eleves');
			}
			foreach($this->eleves as $E){
				$C = array('classe' => $E['classe'], 'rne'=>$this->RNE);
				$html .= $this->modele->storeClasse($C,'sconet_classes');
				$this->modele->storeClasse($C,'classes');
			}

			foreach($this->elevesAnciens() as $id => $E){

				$html .= $this->modele->ancienEleve($id, $E, 'sconet');	
			}
			// tout s'est bien passé ! on valide :
			if( $trans ){
				// tout s'est bien passé ! on valide :
				$this->modele->PDO->commit();
			} else {
				// tout s'est bien passé ! On demande à l'utilisateur de valider
				$this->modele->PDO->rollBack();
			}
			// sinon une exception a eu lieu ...
		}
		catch(Exception $e){
			// ... on annule les modifications
			$this->modele->PDO->rollBack();
			$html .= utf8_encode("<h2 class='erreur'>La base n'a pu &ecirc;tre mise &agrave; jour !</h2>");
			//$html .= utf8_encode("<p class='ui-state-error'>".$e->getMessage()."</p>");
			$html .= "<p class='ui-state-error'>Un probl&egrave;me est survenu avec les donn&eacute;es de l'&eacute;l&egrave;ve ".$E['prenom']." ".htmlentities($E['nom'])." lors de son enregistrement dans la base</p>";
			$html .= $e->getMessage();
		}
		
		//__LXF__(__CLASS__."::".__FUNCTION__, $args, $html );
		return $html;	
	}
	
	/**
	* Retourne une chaine normalisee (pas d'accent, pas d'espace, pas de quot,...) de longueur maximale donnée
	* @param string $string
	* @param int $long = 50
	* @return string
	*/
	private function normalize ($string, $long = 50) {
		$args = get_defined_vars();
    	$string=strtolower($string);
		$old="ÁÀÂÄÃÅÇÉÈÊËÍÏÎÌÑÓÒÔÖÕÚÙÛÜÝáàâäãåçéèêëíìîïñóòôöõúùûüýÿ-'";
		$new="AAAAAACEEEEIIIINOOOOOUUUUYaaaaaaceeeeiiiinooooouuuuyy000";
		$string=str_replace(' ','',$string);
		$output=substr(strtolower(strtr($string,$old,$new)),0,$long);
		$$output=str_replace('0','',$output);
		if(strstr($output,"&")){
			$output = preg_replace('#&([A-za-z])(?:acute|cedil|caron|circ|grave|orn|ring|slash|th|tilde|uml);#', '\1', $string);
			$output = str_replace("&#039;","",$output);
		    $output = str_replace("&amp;#039;","",$output);
		    $output = str_replace("&amp;#39;","",$output);
		    $output = str_replace(" ","",$output);
		}
		


		//__LXF__(__CLASS__."::".__FUNCTION__, $args, $string );
		return $output;
	}
	
	/**
	* Retourne le login d'un eleve pnom unique
	* @param string $prenom
	* @param string $nom
	* @param int $i
	* @return string
	*/
	private function getLogin($prenom,$nom){
		$args = get_defined_vars();

		$nom = $this->normalize ($nom, 10);
		$login = strtolower($prenom[0].$nom);
		$sql= "SELECT login  FROM (";
		$sql.="SELECT login FROM sconet where rne='".$this->RNE."' ";
		$sql.=" UNION ";
		$sql.=" SELECT login FROM profs ) AS temp1 where login LIKE '".$login."%'";
		$trouve = false;
		
		$res = $this->modele->PDO->query($sql);
		$dat = $res->fetchAll(PDO::FETCH_ASSOC);
		$tab_login[] = 0;
		
		foreach ($dat as $log) {
			$trouve = true;
			$num = str_replace($login,'',$log["login"]);
			$tab_login[] = $num;
		}
		if($trouve){
			$max = max($tab_login)+1;
			$return = $login.$max;
		}
		else{
			$return = $login;
		}

		//__LXF__(__CLASS__."::".__FUNCTION__, $args, $return );
		return $return;
	} 
	
	public function reponse(){
		$args = get_defined_vars();

		// Profil;Etat;Identifiant;Sconet;Nom;PrÈnom;Niveau;Mef;Classe
		$br = "<br/>"; $hr = "<hr/>";
		ob_start();
		foreach($this->eleves as $id => $eleve){
			echo $id.$br;
			echo $eleve['id_cas'].$br;
			echo $eleve['nom'].$br;
			echo $eleve['prenom'].$br;
			echo $eleve['classe'].$br.$hr;
		}
		$reponse = ob_get_contents();
		ob_end_clean();

		//__LXF__(__CLASS__."::".__FUNCTION__, $args, $reponse );
		return $reponse;
	}
			
}

?>